import 'dart:io';

import 'package:args/args.dart';
import 'package:get_cli/file_model.dart';
import 'package:get_cli/sample_interface.dart';
import 'package:get_cli/structure.dart';
import 'package:http/http.dart';
import 'package:recase/recase.dart';
import 'models/get_controller.dart';
import 'models/get_binding.dart';
import 'models/get_route.dart';
import 'models/get_view.dart';

/// This is an defaultGenerator which takes [defaultStructure] as an parameter.
Generator defaultGenerator = Generator(currentStructure: defaultStructure);

/// Generator takes an [Structure] object, to specify where to create .dart files
class Generator {
  final Structure currentStructure;

  Generator({
    this.currentStructure,
  }) : assert(currentStructure != null);

  /// This method creates the file in expected path and writes inside a file as expected.
  ///     example: get widget --name LoginPage
  ///
  Future handleFileCreate(ArgResults results) async {
    await defaultGenerator.createFile(
        fileModel: FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: results['extraFolder'],
      ),
      commandName: results.command.name,
    ));
  }

  Future<File> getOnlineModel(String path, String url) async {
    final file = File(path);

    await file.writeAsString(BindingSample().file(await read(url)));
    // BindingSample().file(await read('https://example.com/foobar.txt')));
    return file;
  }

  /// This function only handles file creation
  Future<File> createFile({FileModel fileModel, Sample content}) async {
    var reCase = ReCase(fileModel.result);
    var _file = await File(fileModel.path + '.dart').create(recursive: true);

    await _file.writeAsString(content.file(reCase.pascalCase));

    print(
        'file created succesfully with name :${fileModel.result} at path: ${fileModel.path}');
    return _file;
  }

  Future<void> createPage(ArgResults results) async {
    FileModel _fileModel = FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: true,
      ),
      commandName: results.command.name,
    );
    ReCase reCase = ReCase(_fileModel.result);

    /// Create module of files
    File _controller = await File(_fileModel.path + "_controller.dart")
        .create(recursive: true);
    await _controller.writeAsString(ControllerSample().file(reCase.pascalCase));
    File _binding =
        await File(_fileModel.path + "_binding.dart").create(recursive: true);

    await _binding.writeAsString(BindingSample().file(reCase.pascalCase));

    // File _route =
    //     await File(_fileModel.path + "_route.dart").create(recursive: true);
    // await _route.writeAsString(RouteSample().file(reCase.pascalCase));

    File _view =
        await File(_fileModel.path + "_view.dart").create(recursive: true);
    await _view.writeAsString(GetViewSample().file(reCase.pascalCase));

    print(reCase.pascalCase + " Page created succesfully.");
  }

  Future<void> createProject(ArgResults results) async {
    FileModel _fileModel = FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: true,
      ),
      commandName: results.command.name,
    );
    ReCase reCase = ReCase(_fileModel.result);

    /// Create module of files
    File _controller = await File(_fileModel.path + "_controller.dart")
        .create(recursive: true);
    await _controller.writeAsString(ControllerSample().file(reCase.pascalCase));
    File _binding =
        await File(_fileModel.path + "_binding.dart").create(recursive: true);

    await _binding.writeAsString(BindingSample().file(reCase.pascalCase));

    File _route =
        await File(_fileModel.path + "_route.dart").create(recursive: true);
    await _route.writeAsString(RouteSample().file(reCase.pascalCase));

    File _view =
        await File(_fileModel.path + "_view.dart").create(recursive: true);
    await _view.writeAsString(GetViewSample().file(reCase.pascalCase));

    await createRoute(results);

    await createMain(results);

    print(reCase.pascalCase + " Page created succesfully.");
  }

  Future<void> createRoute(ArgResults results) async {
    FileModel _fileModel = FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: false,
      ),
      commandName: results.command.name,
    );
    ReCase reCase = ReCase(_fileModel.result);

    File _route =
        await File(_fileModel.path + "_route.dart").create(recursive: true);
    await _route.writeAsString(RouteSample().file(reCase.pascalCase));

    print(reCase.pascalCase + " Page created succesfully.");
  }

  Future<void> createMain(ArgResults results) async {
    FileModel _fileModel = FileModel(
      result: results['name'],
      path: Structure.getPathWithName(
        defaultStructure.getPathByKey(results.command.name),
        ReCase(results['name']).snakeCase,
        createWithWrappedFolder: true,
      ),
      commandName: results.command.name,
    );
    ReCase reCase = ReCase(_fileModel.result);

    File _route =
        await File(_fileModel.path + "_main.dart").create(recursive: true);
    await _route.writeAsString(RouteSample().file(reCase.pascalCase));

    print(reCase.pascalCase + " Main created succesfully.");
  }
}

/// This function takes the results from the [ArgParser] and act as expected.
Future<void> generate({
  ArgResults results,
  Generator generator,
}) async {
  switch (results.command.name) {
    case "page":
      await generator.createPage(results);
      break;
    case "widget":
      await generator.handleFileCreate(results);
      break;
    case "data":
      await generator.handleFileCreate(results);
      break;
    case "model":
      await generator.handleFileCreate(results);
      break;
    case "view":
      await generator.handleFileCreate(results);
      break;
    case "route":
      await generator.createRoute(results);
      break;
    case "project":
      await generator.createProject(results);
      break;
    case "controller":
      await generator.handleFileCreate(results);
      break;
  }
}
